<?php
/*

Copyright (c)2002-2003 DuckCorp(tm) and RtpNet(tm)



This file is part of LMS.

LMS is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

LMS is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with LMS; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/



/**
* User Type : unknown
*
* @const LMS_UTYPE_UNKNOWN
*/
define('LMS_UTYPE_UNKNOWN', 0);
/**
* User Type : admin
*
* @const LMS_UTYPE_ADMIN
*/
define('LMS_UTYPE_ADMIN', 1);
/**
* User Type : common
*
* @const LMS_UTYPE_COMMON
*/
define('LMS_UTYPE_COMMON', 2);
/**
* User Type : service
*
* @const LMS_UTYPE_SERVICE
*/
define('LMS_UTYPE_SERVICE', 3);

/**
* LMS User Manager CLass
*
* @access	public
*/
class LmsUserManager extends LmsObjectManager
{

	/**
	* Constructor
	*
	* @access	public
	* @param	object DftkLdapConnection	&$ldap_connection
	*/
	function LmsUserManager(&$tracemgr, &$ldap_connection)
	{
		global		$config;

		LmsObjectManager::LmsObjectManager(&$tracemgr, &$ldap_connection);

		$this->_dn_attr = "uid";
		$this->_root_dn = "ou=Users,".$config[base];
		$this->_propagation_key_list = array('memberUid', 'ownerUID', 'adminUID');
		$this->_check_key_list = array('objectclass', 'homephone', 'mobile', 'mail');
		$this->_base_objectclasses = array('top', 'posixAccount', 'shadowAccount');


		$this->_register_attr("UID", "uid", LMS_ATYPE_STR, 30, 1, true);
		$this->_register_attr("Common Name", "cn", LMS_ATYPE_STR, 30, 1, true);
		$this->_register_attr("UID Number", "uidnumber", LMS_ATYPE_INT, null, 1, true);
		$this->_register_attr("GID Number", "gidnumber", LMS_ATYPE_INT, null, 1, true);
		$this->_register_attr("Surname", "sn", LMS_ATYPE_STR, 30, 1, false);
		$this->_register_attr("Given Name", "givenname", LMS_ATYPE_STR, 30, 1, false);
		$this->_register_attr("Nickname", "nn", LMS_ATYPE_STR, 30, 1, false);
		$this->_register_attr("Home Directory", "homedirectory", LMS_ATYPE_STR, 60, 1, false);
		$this->_register_attr("Password", "userpassword", LMS_ATYPE_STR, 100, 1, false);
		$this->_register_attr("Shell", "loginshell", LMS_ATYPE_STR, 60, 1, false);
		$this->_register_attr("Gecos", "gecos", LMS_ATYPE_STR, 100, 1, false);
		$this->_register_attr("Description", "description", LMS_ATYPE_STR, 300, 1, false);
		$this->_register_attr("Days since Jan 1, 1970 that password was last changed", "shadowlastchange", LMS_ATYPE_INT, null, 1, false, false);
		$this->_register_attr("Days before Password may be changed", "shadowmin", LMS_ATYPE_INT, null, 1, false);
		$this->_register_attr("Days before Password must be changed", "shadowmax", LMS_ATYPE_INT, null, 1, false);
		$this->_register_attr("Days before password is to expire that user is warned", "shadowwarning", LMS_ATYPE_INT, null, 1, false);
		$this->_register_attr("Days after password expires that account is disabled", "shadowinactive", LMS_ATYPE_INT, null, 1, false);
		$this->_register_attr("Days since Jan 1, 1970<BR>that account is disabled", "shadowexpire", LMS_ATYPE_INT, null, 1, false, false);
		$this->_register_attr("Account is disactivated", "isdisactivated", LMS_ATYPE_BOOL, null, 1, false);
		$this->_register_attr("Contribution is paid", "contributionispaid", LMS_ATYPE_BOOL, null, 1, false);
		$this->_register_attr("Forbid profile modification", "profilemodifforbiden", LMS_ATYPE_BOOL, null, 1, false);
		$this->_register_attr("Birthday", "birthday", LMS_ATYPE_STR, 30, 1, false);
		$this->_register_attr("Occupation", "occupation", LMS_ATYPE_STR, 60, 1, false);
		$this->_register_attr("Room Number", "roomnumber", LMS_ATYPE_INT, null, 1, false);
		$this->_register_attr("Home Address", "homepostaladdress", LMS_ATYPE_STR, 30, 1, false);
		$this->_register_attr("Home Telephone", "homephone", LMS_ATYPE_TELNO, null, 1, false);
		$this->_register_attr("Mobile Telephone", "mobile", LMS_ATYPE_TELNO, null, 1, false);
		$this->_register_attr("Mail Alias", "mailalias", LMS_ATYPE_STR, 30, 1, false);
		$this->_register_attr("Mails", "mail", LMS_ATYPE_STR, 30, 10, false);
		$this->_register_attr("icq uin", "icq", LMS_ATYPE_INT, null, 1, false);
		$this->_register_attr("Yahoo Messenger nick", "ym", LMS_ATYPE_STR, 30, 1, false);
		$this->_register_attr("AOL Instant Messenger nick", "aim", LMS_ATYPE_STR, 30, 1, false);
		$this->_register_attr("Favourite Drink", "drink", LMS_ATYPE_STR, 30, 1, false);
		
		register_shutdown_function(array(&$this, "_LmsUserManager"));
	}

	/**
	* Destructor
	*
	* @access	public
	*/
	function _LmsUserManager()
	{
	}

	/**
	* Give User Type
	*
	* @access	public
	* @return	object DftkDaTrace	$r			Trace
	*/
	function &get_user_type()
	{
		$r =& $this->_tracemgr->create_trace();

		if (!is_array($this->_attrs['objectclass']['value']))
			$r->add_event('lms-api_usertypeno');
		elseif (in_array('lmsAdminUser', $this->_attrs['objectclass']['value']))
			$r->set_result('user_type', LMS_UTYPE_ADMIN);
		elseif (in_array('lmsCommonUser', $this->_attrs['objectclass']['value']))
			$r->set_result('user_type', LMS_UTYPE_COMMON);
		elseif (in_array('lmsServiceUser', $this->_attrs['objectclass']['value']))
			$r->set_result('user_type', LMS_UTYPE_SERVICE);
		else
			$r->add_event('lms-api_usertypebadt');

		return $r;
	}

	/**
	* Find a User with his UID
	*
	* @access	public
	* @param	string			$uid			UID of the User
	* @return	object DftkDaTrace	$r			Trace
	*/
	function &find_by_uid($uid)
	{
		$r =& $this->_lc->search($this->_root_dn, "uid=".$uid, $this->_ent);
		if ($r->has_error)
			$r->add_event('lms-api_userfinduse', $uid);
		else
		{
			$nb = $r->get_result('count');

			if ($nb!=0)
			{
				$this->_entry_to_attrs(0, &$r);
				$r->set_result('count', $this->_ent_get_count());
			}
			else
				$r->add_event('lms-api_userfindunf', $uid);

			if ($n>0)
				$r->add_event('lms-api_probconsit');
		}

		return $r;
	}

	/**
	* Find a User with his UID Number
	*
	* @access	public
	* @param	integer			$uidnumber		UID Number of the User
	* @return	object DftkDaTrace	$r			Trace
	*/
	function &find_by_uidnumber($uidnumber)
	{
		$r =& $this->_lc->search($this->_root_dn, "uidNumber=".$uid, $this->_ent);
		if ($r->has_error)
			$r->add_event('lms-api_userfindunse', $uid);
		else
		{
			$nb = $r->get_result('count');

			if ($nb!=0)
			{
				$this->_entry_to_attrs(0, &$r);
				$r->set_result('count', $this->_ent_get_count());
			}
			else
				$r->add_event('lms-api_userfindunnf', $uid);

			if ($n>0)
				$r->add_event('lms-api_probconsit');
		}

		return $r;
	}

	/**
	* Give the UID corresponding to an UID Number
	*
	* @access	public
	* @param	integer			$uidnumber		UID Number of the User
	* @return	object DftkDaTrace	$r			Trace
	*/
	function &get_uid_from_uidnumber($uidnumber)
	{
		$r =& $this->_lc->search($this->_root_dn, "uidNumber=".$uid, $this->_ent, array('uid'));
		if ($r->has_error)
			$r->add_event('lms-api_userconvunuse', $uidnumber);
		else
		{
			$nb = $r->get_result('count');

			if ($nb!=0)
			{
				$this->_entry_to_attrs(0);
				$r->set_result('uid', $this->_attrs['uid']['value']);
			}
			else $r->add_event('lms-api_userconvununf', $uidnumber);

			if ($n>0)
				$r->add_event('lms-api_probconsit');
		}

		return $r;
	}

	/**
	* Give the UID Number corresponding to an UID
	*
	* @access	public
	* @param	string			$uid			UID Number of the User
	* @return	object DftkDaTrace	$r			Trace
	*/
	function &get_uidnumber_from_uid($uid)
	{
		$r =& $this->_lc->search($this->_root_dn, "uid=".$uid, $this->_ent, array('uidnumber'));
		if ($r->has_error)
			$r->add_event('lms-api_userconvuunse', $uid);
		else
		{
			$nb = $r->get_result('count');

			if ($nb!=0)
			{
				$this->_entry_to_attrs(0);
				$r->set_result('uid', $this->_attrs['uidnumber']['value']);
			}
				$r->add_event('lms-api_userconvuunnf', $uid);

			if ($n>0)
				$r->add_event('lms-api_probconsit');
		}

		return $r;
	}

	/**
	* Get a User list
	*
	* Always fetched attributs : objectclass, uid, cn, sn, givenname, nn
	*
	* @access	public
	* @param	integer			$c			User Type to list (0 => every users)
	* @param	array			$attr			Complementary attributs to fetch
	* @param	string			$crit			Alternative criterias
	* @return	object DftkDaTrace	$r			Trace
	*/
	function &get_list($c, $attr = array(), $crit = "")
	{
		if (!is_array($attr))
			$attr = array();
		$attr = array_merge($this->_check_key_list, $attr);
		$attr = array_merge(array('uid', 'uidnumber', 'cn', 'sn', 'givenname', 'nn'), $attr);
		if (!$crit)
			$crit = "uid=*";
		$badt = false;
		switch ($c)
		{
		case LMS_UTYPE_UNKNOWN:
			$r =& $this->_lc->search($this->_root_dn, $crit, $this->_ent, $attr);
			break;
		case LMS_UTYPE_ADMIN:
			$r =& $this->_lc->search("ou=AdminUsers,".$this->_root_dn, $crit, $this->_ent, $attr, false);
			break;
		case LMS_UTYPE_COMMON:
			$r =& $this->_lc->search("ou=CommonUsers,".$this->_root_dn, $crit, $this->_ent, $attr, false);
			break;
		case LMS_UTYPE_SERVICE:
			$r =& $this->_lc->search("ou=ServiceUsers,".$this->_root_dn, $crit, $this->_ent, $attr, false);
			break;
		default:
			$bast = true;
			$r =& $this->_tracemgr->create_trace();
			$r->add_event('lms-api_userlistbadt', $crit);
		}

		if (!$badt)
		{
			if ($r->has_error())
				$r->add_event('lms-api_userlistse', array($c, $crit));
			else
			{
				$this->_entry_to_attrs(0, &$r);
				$r->set_result('count', $this->_ent_get_count());
			}
		}

		return $r;
	}

	/**
	* Give the first unused UID Number
	*
	* @access	public
	* @param	integer			$c			User Type to list (0 => every users)
	* @return	object DftkDaTrace	$r			Trace
	*/
	function &get_first_free_uidnumber($c)
	{
		$r =& $this->_tracemgr->create_trace();

		switch ($c)
		{
		case LMS_UTYPE_ADMIN:
		case LMS_UTYPE_COMMON:
			$min = 10000;
			$max = 65535;
			break;
		case LMS_UTYPE_SERVICE:
			$min = 5000;
			$max = 10000;
			break;
		case LMS_UTYPE_UNKNOWN:
		default:
			$r->add_event('lms-api_userfreeunbadt');
		}

		if (!$r->has_error())
		{
			$r =& $this->_lc->search($this->_root_dn, "uid=*", $this->_ent, array('uidnumber'), false);

			if ($r->has_error())
				$r->add_event('lms-api_userfreeunse');
			else
			{
				$z = array();
				$ret = 0;
				for($i=0; $i<$this->_ent_get_count(); $i++)
				{
					$this->_entry_to_attrs($i);
					$z[$i] = $this->_attrs['uidnumber']['value'];
				}
				for ($i=$min; $i<=$max; $i++)
				{
					if (!in_array($i, $z))
					{
						$ret = $i;
						break;
					}
				}

				if ($ret!=0)
					$r->set_result('first_free_uidnimber', $ret);
				else
					$r->add_event('lms-api_userfreeunnf', $c);
			}
		}

		return $r;
	}

	/**
	* Do what need to be done before applying changes (OVERLOAD)
	*
	* @access	private
	*/
	function _pre_commit_changes()
	{
		LmsObjectManager::_pre_commit_changes();

		$r =& $this->get_user_type();
		$user_type = $r->get_result('user_type');
		if ($user_type==LMS_UTYPE_SERVICE)
		{
			if ($this->_attrs['uid']['changed'])
				$this->_set_attr_value('cn', $this->_attrs['uid']['value']);
			$this->_set_attr_value('initials', null);
		}
		else
		{
			if ($this->_attrs['sn']['changed'] || $this->_attrs['givenname']['changed'])
			{
				$this->_set_attr_value('cn', $this->_attrs['sn']['value']." ".$this->_attrs['givenname']['value']);
				$this->_set_attr_value('initials', strtoupper(substr($this->_attrs['sn']['value'], 0, 1)." ".substr($this->_attrs['givenname']['value'], 0, 1)));
			}
		}

		if ($this->_attrs['userpassword']['changed'])
		{
			if (substr($this->_attrs['userpassword']['value'], 0, 7)!="{crypt}")
			{
				$this->_set_attr_value('userpassword', "{crypt}".dftk_unix_crypt($this->_attrs['userpassword']['value']));
			}
		}

		if (!$this->_attrs['homedirectory']['value'])
			$this->_set_attr_value('homedirectory', "/home/".$this->_attrs['uid']['value']);

		if (!$this->_attrs['uidnumber']['value'])
			$this->_set_attr_value('uidnumber', $this->get_first_free_uidnumber($user_type));
	}

	/**
	* Additionnal Check (OVERLOAD)
	*
	* @access	private
	* @return	boolean		$ok		True if successful
	*/
	function _check_changes($mod = array())
	{
		$r =& LmsObjectManager::_check_changes($mod);
		
		$r2 =& $this->get_user_type();
		$r->merge_traces($r2);
		if (!$r->has_error())
		{
			$user_type = $r->get_result('user_type');
			if ($user_type==LMS_UTYPE_ADMIN)
			{
				if (!$this->_attrs['homephone']['value'] && !$this->_attrs['mobile']['value'])
					$r->add_event('lms-api_userchkchgmv1');
				if (count($this->_attrs['mail']['value'])==0)
					$r->add_event('lms-api_userchkchgmv2');
			}
		}

		return $r;
	}

	/**
	* Return a sub DN if needed (OVERLOAD)
	*
	* @access	public
	* @return	string
	*/
	function _get_sub_dn()
	{
		$r =& $this->get_user_type();
		$user_type = $r->get_result('user_type');

		switch($user_type)
		{
		case LMS_UTYPE_SERVICE:
			return "ou=ServiceUsers";
		case LMS_UTYPE_ADMIN:
			return "ou=AdminUsers";
		default:
			return "ou=CommonUsers";
		}
	}

	/**
	* Change User Type
	*
	* @access	public
	* @param	integer			$new_user_type		Type of User
	* @return	object DftkDaTrace	$r			Trace
	*/
	function &set_user_type($new_user_type)
	{
		$r =& $this->_tracemgr->create_trace();

		if ($this->_ent_get_count() != 0)
		{
			$r2 =& $this->get_user_type();
			$r->merge_traces($r2);
			if ($r->has->error())
				$r->add_event('lms-api_usersettse');
			else
			{
				$user_type = $r->get_result('user_type');

				if ($new_user_type==$user_type)
					$r->add_event('lms-api_usersettntd');
				if (($user_type==LMS_UTYPE_SERVICE) || ($new_user_type==LMS_UTYPE_SERVICE))
					$r->add_event('lms-api_usersettbadc');
			}
		}

		if (!$r->has_error())
		{
			$r2 =& $this->_set_user_type($new_user_type);
			$r->merge_traces($r2);
		}

		return $r;
	}

	/**
	* Change User Type (internal use)
	*
	* @access	private
	* @param	integer			$new_user_type		Type of User
	* @return	object DftkDaTrace	$r			Trace
	*/
	function &_set_user_type($new_user_type = null, $set_changed = true)
	{
		$r =& $this->_tracemgr->create_trace();

		if (!$new_user_type)
		{
			$r =& $this->get_user_type();
			$new_user_type = $r->get_result('user_type');
		}

		switch($new_user_type)
		{
		case LMS_UTYPE_ADMIN:
			$this->_attrs['objectclass']['value'] = array('lmsAdminUser');
			$this->_activate_attrs(array('nn', 'initials', 'roomnumber', 'homepostaladdress', 'hometelephonenumber', 'mobiletelephonenumber', 'mail', 'favouritedrink', 'suscribedservice', 'icq', 'ym', 'aim', 'occupation', 'mailalias', 'birthday'));
			$this->_desactivate_attrs(array('contributionispaid', 'profilemodifforbiden'));
			break;
		case LMS_UTYPE_COMMON:
			$this->_attrs['objectclass']['value'] = array('lmsCommonUser');
			$this->_activate_attrs(array('nn', 'initials', 'roomnumber', 'homepostaladdress', 'hometelephonenumber', 'mobiletelephonenumber', 'mail', 'favouritedrink', 'suscribedservice', 'contributionispaid', 'icq', 'ym', 'aim', 'occupation', 'mailalias', 'profilemodifforbiden', 'birthday'));
			break;
		case LMS_UTYPE_SERVICE:
			$this->_attrs['objectclass']['value'] = array('lmsServiceUser');
			$this->_desactivate_attrs(array('nn', 'initials', 'roomnumber', 'homepostaladdress', 'hometelephonenumber', 'mobiletelephonenumber', 'mail', 'favouritedrink', 'suscribedservice', 'contributionispaid', 'icq', 'ym', 'aim', 'occupation', 'mailalias', 'profilemodifforbiden', 'birthday'));
			break;
		default:
			$r->add_event('lms-api_usersettbadt');
		}

		if ($set_changed)
			$this->_attrs['objectclass']['changed'] = true;

		return $r;
	}

	/**
	* Do Object need a renaming before saving changes ? (OVERLOAD)
	*
	* @access	private
	* @return	boolean
	*/
	function _need_renaming()
	{
		return $this->_attrs['objectclass']['changed'];
	}

	/**
	* Convert an LDAP Entry to Attributs (OVERLOAD)
	*
	* @access	private
	* @param	integer		$i		Number of the Entry to convert
	*/
	function _entry_to_attrs($i, $r = null)
	{
		LmsObjectManager::_entry_to_attrs($i, &$r);

		$this->_set_user_type(null, false);
	}

}

?>
